package ch.inf3.horizoncut.data;

public class DisplayCalculator {

    final int width;
    final int height;
    final double minHorizAngle;
    final double maxHorizAngle;
    final double eyeHeight;

    public DisplayCalculator(int width, int height, double minHorizAngle, double maxHorizAngle, double eyeHeight) {
        this.width = width;
        this.height = height;
        this.minHorizAngle = minHorizAngle;
        this.maxHorizAngle = maxHorizAngle;
        this.eyeHeight = eyeHeight;
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public double getMinHorizAngle() {
        return minHorizAngle;
    }

    public double getMaxHorizAngle() {
        return maxHorizAngle;
    }

    public double getEyeHeight() {
        return eyeHeight;
    }

    public double getAngleStep() {
        return (maxHorizAngle - minHorizAngle) / width;
    }

    public boolean isVisible(double bearing) {
        return (bearing <= maxHorizAngle && bearing >= minHorizAngle);
    }

    /**
     *
     * @param x
     * @return Bearing in degrees
     */
    public double getBearingAtX(int x) {
        return (x * getAngleStep()) + minHorizAngle;
    }

    /**
     *
     * @param bearing Bearing in degrees
     * @return
     */
    public int getXAtBearing(double bearing) {
        return (int) ((bearing - minHorizAngle) / getAngleStep());
    }

    public double calculateVisibilityAngle(double targetVisibleHeight, int targetDistance) {
        if (eyeHeight == targetVisibleHeight) {
            return 90.0;
        } else if (eyeHeight < targetVisibleHeight) {
            double op = targetVisibleHeight - eyeHeight;
            double ad = targetDistance;
            return Math.toDegrees(Math.atan(op / ad)) + 90;
        } else {
            return Math.toDegrees(Math.atan(targetDistance / (eyeHeight - targetVisibleHeight)));
        }
    }
}
