package ch.inf3.horizoncut.data;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class TileMap {

    private final Map<Integer, Map<Integer, Tile>> tileByLatLon = new HashMap<>();

    public TileMap(Collection<Tile> tiles) {
        for (Tile tile : tiles) {
            add(tile);
        }
    }

    private void add(Tile tile) {
        insertIntoCache(tile);
    }

    private void insertIntoCache(Tile tile) {
        tileByLatLon.putIfAbsent(tile.lat(), new HashMap<>());
        tileByLatLon.get(tile.lat()).put(tile.lon(), tile);
    }

    public Tile getByLatLon(int lat, int lon) {
        if (!tileByLatLon.containsKey(lat) || !tileByLatLon.get(lat).containsKey(lon)) {
            add(new Tile.EmptyTile(lat, lon, null));
        }
        return tileByLatLon.get(lat).get(lon);
    }

}
